from typing import Any
from enum import Enum
from redis import Redis, ConnectionPool

from ..context import Context
from ..model import Service


class RedisKey(Enum):
    session: int = ...


class GlobalData:
    redis_pool: Any


class RedisServ(Service):
    ctx: Context
    redis: Redis
    def __init__(self, ctx: Context) -> None: ...


global_data: GlobalData = ...


def init(host: str, port: int=..., db: int=...): ...
def processor(ctx: Context): ...
def session() -> Redis: ...



"""
Dom Element
"""
# __pragma__ ('skip')
import json
import os


def buildjs(srcpath: str, distpath: str=...) -> None: ...

# __pragma__ ('noskip')


def tofixed(num, *, precision) -> str: ...


def uncapitalize_name(name: str) -> str: ...


class Div:
    JSVOID: str = ...
    def __init__(self, *children, **attrs) -> None: ...
    def _dump_head(self): ...
    def dumps(self) -> str: ...


def HtmlPage(*nodes, title=..., https=False) -> Div: ...
def FlexRow(*nodes, Id=None) -> Div: ...
def FlexItem(*nodes, Id=None) -> Div: ...
def Button(text, *, Onclick:str, primary=False, warn=False, Id=None) -> Div: ...
def TextInput(title, *, Id, Name=None, Value=None, Type=...) -> Div: ...
def FileInput(title, *, Id, Name=None) -> Div: ...
def SelectInput(title, *options, Id, Name=None, Value=None, Onchange=None) -> Div: ...
def SelectOption(text, *, Value, selected=False) -> Div: ...
def TextArea(title, *, Id, Name=None, Rows:int=...) -> Div: ...
def Link(text, *, Url, Id=None) -> Div: ...
def Module(From:str, Import:str, As:str, Path=...) -> Div: ...


# Transcrypt only
class Kit:
    @staticmethod
    def ajax(method, url, data=None, headers=None, contentType=None, onsuccess=None, onerror=None, **kw): ...
    @staticmethod
    def request(*, method, url, data=None, headers=None, json=False, onsuccess=None, onerror=None): ...
    @staticmethod
    def get_json(url, data, onsuccess): ...
    @staticmethod
    def select(selector): ...
    @staticmethod
    def onload(handler): ...
    @staticmethod
    def param(data): ...
    @staticmethod
    def parseJSON(text): ...
    @staticmethod
    def stringifyJSON(data): ...
    @staticmethod
    def print(obj): ...
    @staticmethod
    def alert(text, ondone=None): ...
    @staticmethod
    def confirm(text, onconfirm=None, oncancel=None): ...
    @staticmethod
    def reload(): ...
    @staticmethod
    def goto(url): ...
    @staticmethod
    def form_data(*selectors): ...


# Transcrypt only
class Storage:
    def __init__(self, name, expire=False) -> None: ...
    def get(self): ...
    def set(self, value): ...



"""
Web application
(from lessweb)
"""
from datetime import datetime
import itertools
import json
import logging
import os
import re
import traceback
from types import GeneratorType
from typing import List, Any, Callable, Type, Dict
from urllib.parse import splitquery, urlencode
from io import BytesIO
from contextlib import contextmanager

from lessweb.webapi import NeedParamError, BadParamError, NotFoundError, HttpStatus
from lessweb.webapi import http_methods
from lessweb.context import Context
from lessweb.model import fetch_param, ModelToDict
from lessweb.storage import Storage
from lessweb.utils import eafp, re_standardize, makedir
from lessweb.bridge import Bridge, assert_valid_bridge
from lessweb.garage import Jsonizable, BaseBridge, JsonToJson


__all__ = [
    "Interceptor", "Mapping", "interceptor", "Application",
]


class Interceptor:
    def __init__(self, pattern, method, dealer, patternobj) -> None: ...

class Mapping:
    def __init__(self, pattern, method, dealer, doc, patternobj, view, querynames) -> None: ...


def build_controller(dealer): ...


def interceptor(dealer): ...


class Application(object):
    def __init__(self, encoding: str='utf-8', debug: bool=True) -> None: ...
    def add_interceptor(self, pattern: str, method: str, dealer: Callable) -> None: ...
    def add_bridge(self, bridge: Type[Bridge]) -> None: ...
    def add_mapping(self, pattern: str, method: str, dealer: Callable, view=None) -> None: ...
    def add_connect_interceptor(self, pattern: str, dealer: Callable) -> None: ...
    def add_connect_mapping(self, pattern: str, dealer: Callable, view=None) -> None: ...
    def add_delete_interceptor(self, pattern: str, dealer: Callable) -> None: ...
    def add_delete_mapping(self, pattern: str, dealer: Callable, view=None) -> None: ...
    def add_get_interceptor(self, pattern: str, dealer: Callable) -> None: ...
    def add_get_mapping(self, pattern: str, dealer: Callable, view=None) -> None: ...
    def add_head_interceptor(self, pattern: str, dealer: Callable) -> None: ...
    def add_head_mapping(self, pattern: str, dealer: Callable, view=None) -> None: ...
    def add_options_interceptor(self, pattern: str, dealer: Callable) -> None: ...
    def add_options_mapping(self, pattern: str, dealer: Callable, view=None) -> None: ...
    def add_post_interceptor(self, pattern: str, dealer: Callable) -> None: ...
    def add_post_mapping(self, pattern: str, dealer: Callable, view=None) -> None: ...
    def add_put_interceptor(self, pattern: str, dealer: Callable) -> None: ...
    def add_put_mapping(self, pattern: str, dealer: Callable, view=None) -> None: ...

    def wsgifunc(self, *middleware) -> Callable: ...
    def run(self, wsgifunc: Callable=None, port:int=8080, homepath:str='', staticpath:str='static') -> None: ...

    def request(self, localpart:str='/', method:str='GET', data=None,
                host:str="0.0.0.0:8080", headers:Dict=None, https:bool=False, env:Dict=None) -> Storage: ...
    def test_get(self, localpart='/', query=None, headers=None, status_code=200, parsejson=True, https=False, env=None): ...
    def test_delete(self, localpart='/', query=None, headers=None, status_code=200, parsejson=True, https=False, env=None): ...
    def test_head(self, localpart='/', query=None, headers=None, status_code=200, parsejson=True, https=False, env=None): ...
    def test_post(self, localpart='/', data=None, headers=None, status_code=200, parsejson=True, https=False, env=None): ...
    def test_put(self, localpart='/', data=None, headers=None, status_code=200, parsejson=True, https=False, env=None): ...



"""lessweb: 用最python3的方法创建web apps"""


#__version__ = '0.2.5'
#__author__ = [
#    'qorzj <inull@qq.com>',
#]
#
#__license__ = "MIT"

from . import application, context, model, storage, webapi

from .application import (
    interceptor as interceptor,
    Application as Application,
)
from .context import (
    Context as Context,
    Request as Request,
    Response as Response,
)
from .model import (
    Model as Model,
    Service as Service,
)
from .storage import (
    Storage as Storage,
    ChainMock as ChainMock,
)
from .bridge import (
    Bridge as Bridge,
)
from .typehint import (
    issubtyping as issubtyping,
    AnySub as AnySub,
)
from .garage import (
    Jsonizable as Jsonizable,
)
from .webapi import (
    NeedParamError as NeedParamError,
    BadParamError as BadParamError,
    NotFoundError as NotFoundError,
    UploadedFile as UploadedFile,
    Cookie as Cookie,
    HttpStatus as HttpStatus,
)
from .utils import (
    _nil as _nil,
    eafp as eafp,
)



from typing import Any, Tuple, Dict, Type, get_type_hints
from contextlib import contextmanager
import json
from pathlib import Path
import pickle
import re
import inspect


def eafp(ask, default): ...


class Nil:
    def __init__(self, value) -> None: ...
    def __bool__(self): ...
    def __eq__(self, other): ...


_nil: Nil = ...
def re_standardize(pattern) -> str: ...
def fields_in_query(query) -> Dict: ...


class StaticDict(dict):
    touched: bool
    def __delitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def update(self, *E, **F) -> None: ...
    def pop(self, *k) -> None: ...


@contextmanager
def static_dict(path): ...
def func_arg_spec(fn)->Dict[str, Tuple[Type, bool]]: ...
def makedir(real_path: str) -> None: ...



from typing import Any, Dict, List, Union


Jsonizable = Union[str, int, float, Dict, List, None]


class BaseBridge(Bridge):
    def __init__(self) -> None: ...
    def to(self)->Any: ...


class JsonToJson(Bridge):
    def __init__(self, source: Jsonizable) -> None: ...
    def to(self) -> Jsonizable: ...



from typing import Callable, Optional, Type, Dict
from abc import ABCMeta

from lessweb.context import Context, Request, Response
from lessweb.webapi import NeedParamError, BadParamError, UploadedFile
from lessweb.typehint import generic_origin
from lessweb.garage import BaseBridge, Jsonizable
from lessweb.bridge import Bridge
from lessweb.utils import func_arg_spec
from lessweb.storage import Storage


class Model(metaclass=ABCMeta):
    def __eq__(self, other): ...
    def __repr__(self): ...


class Service(metaclass=ABCMeta):
    ...


def fetch_service(ctx: Context, service_type: Type): ...
def fetch_model(ctx: Context, model_type: Type[Model]): ...
def fetch_param(ctx: Context, fn: Callable) -> Dict: ...


class ModelToDict(Bridge):
    def __init__(self, source: Model) -> None: ...
    def to(self) -> Jsonizable: ...



from typing import Any
from unittest.mock import Mock


class Storage(dict):
    def __getattr__(self, key): ...
    def __setattr__(self, key, value): ...
    def __delattr__(self, key): ...
    def __repr__(self): ...
    def __sub__(self, other): ...
    @staticmethod
    def of(object) -> 'Storage': ...


class ChainMock:
    def __init__(self, path: str, return_value) -> None: ...
    def join(self, path, return_value) -> 'ChainMock': ...
    def __call__(self, path=None) -> Any: ...



from typing import *
import base64
import os
import hashlib
import time
import requests
import xml.etree.ElementTree as ET
import logging


# 统一下单接口
URL_ACCESS_TOKEN: str = ...
URL_JS_TICKET: str = ...
UNIFIED_ORDER_URL: str = ...
SEND_REDPACK_URL: str = ...


class Weixin:
    appid: str
    appsecret: str
    mchid: str  # (支付必需)商户ID，微信商户平台(pay.weixin.qq.com)->产品中心->开发配置->商户号
    paykey: str  # (支付必需)签名用Key，微信商户平台(pay.weixin.qq.com)->账户中心->账户设置->API安全->密钥设置
    certpath: str  # (退款、发红包等必需)证书所在路径，此路径下需包含apiclient_cert.pem和apiclient_key.pem两个文件
    debug: bool  # 是否打印debug日志
    def __init__(self, *, appid: str, appsecret: str, mchid: str=..., paykey: str=..., certpath: str=..., debug: bool=...) -> None: ...
    @staticmethod
    def make_nonce() -> str: ...
    def create_unified_order_sign(self, params) -> str: ...
    def create_unified_order_xml(self, params) -> str: ...
    def debug_log(self, content) -> None: ...
    def request_prepay_and_get_jspay_sign(self, body, billno, fee, ip, openid, notifyurl) -> Dict: ...
    def send_redpack(self, billno, fee, ip, openid, send_name, wishing, act_name, remark) -> Dict: ...
    def access_token(self) -> Dict: ...
    def js_ticket(self, access_token) -> Dict: ...
    def config_sign(self, js_ticket, url) -> Dict: ...
    def payed(self, data: bytes) -> Dict: ...



from typing import Optional, Dict, List, Union, Any
import cgi
import json

from io import BytesIO
from requests.structures import CaseInsensitiveDict

from lessweb.storage import Storage
from lessweb.webapi import UploadedFile, Cookie, HttpStatus
from lessweb.webapi import header_name_of_wsgi_key, wsgi_key_of_header_name
from lessweb.webapi import parse_cookie, mimetypes
from lessweb.utils import fields_in_query
from lessweb.garage import Jsonizable


class Request:
    def __init__(self) -> None: ...
    def contains_cookie(self, name: str) -> bool: ...
    def get_cookie(self, name: str) -> Optional[str]: ...
    def get_cookienames(self) -> List[str]: ...
    def contains_header(self, name: str) -> bool: ...
    def get_header(self, name: str) -> Optional[str]: ...
    def get_headernames(self) -> List[str]: ...


class Response:
    def __init__(self) -> None: ...
    def set_cookie(self, name:str, value:str, expires:int=None, path:str='/',
                   domain:str=None, secure:bool=False, httponly:bool=False) -> None: ...
    def get_cookie(self, name:str) -> Optional[Cookie]: ...
    def del_cookie(self, name:str) -> None: ...
    def set_status(self, status: HttpStatus) -> None: ...
    def get_status(self) -> HttpStatus: ...
    def set_header(self, name: str, value: Union[str, int]) -> None: ...
    def get_header(self, name: str) -> Optional[str]: ...
    def del_header(self, name: str) -> None: ...
    def get_headernames(self) -> List[str]: ...
    def clear(self) -> None: ...
    def send_access_allow(self, allow_headers: List[str]=None) -> None: ...
    def send_allow_methods(self, methods: List[str]): ...
    def send_redirect(self, location: str) -> None: ...
    def send_content_type(self, mimekey='html', encoding: str=''): ...


class Context(object):
    from lessweb.application import Application
    app_stack: List
    app: Application
    view: Any
    environ: Dict
    env: Dict
    host: str
    protocol: str
    homedomain: str
    homepath: str
    home: str
    realhome: str
    ip: str
    method: str
    path: str
    query: str
    fullpath: str
    request: Request
    response: Response
    def __init__(self, app=None) -> None: ...
    def __call__(self): ...
    def set_param(self, realname, realvalue) -> None: ...
    def get_param(self, realname, default=None) -> Any: ...
    def set_alias(self, realname, queryname) -> None: ...
    def is_json_request(self) -> bool: ...
    def body_data(self) -> bytes: ...
    def get_input(self, queryname, default=None)->Jsonizable: ...
    def get_inputs(self)->Dict[str, Jsonizable]: ...



from typing import Any, overload, Type, TypeVar, get_type_hints, Iterable, List
from urllib.parse import quote
from enum import Enum

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.orm.session import Session
from sqlalchemy.ext.declarative import as_declarative, declared_attr

from ..context import Context
from ..model import Service

__all__ = ["global_data", "DbModel", "DbServ", "init", "processor", "create_all", "make_session",
           "cast_model", "cast_models"]


class DatabaseKey(Enum):
    session: int = ...


class GlobalData:
    db_session_maker: Any
    db_engine: Any
    autocommit: bool


class DbServ(Service):
    ctx: Context
    db: Session
    def __init__(self, ctx: Context) -> None: ...


global_data: GlobalData = ...


@as_declarative()
class DbModel(object):
    @declared_attr
    def __tablename__(cls) -> str: ...
    def __repr__(self): ...


@overload
def init(*, dburi, echo=True, autoflush=True, autocommit=False): ...
@overload
def init(*, protocol, username, password, host, port:int, database, echo=True, autoflush=True, autocommit=False): ...
def init(*, protocol=None, username=None, password=None, host=None, port:int=None, database=None, dburi=None,
         echo:bool=True, autoflush=True, autocommit=False) -> None: ...


def processor(ctx: Context): ...
def create_all(*DbModelClass) -> None: ...


class make_session:
    session: Session
    def __init__(self) -> None: ...
    def __enter__(self) -> Session: ...
    def __exit__(self, type_, value, traceback): ...


T = TypeVar('T')
U = TypeVar('U')


def cast_model(modelCls: Type[T], tblObjs) -> T: ...
def cast_models(modelCls: Type[T], tblObjsList) -> List[T]: ...



from typing import Optional, Dict, Tuple
from http.cookies import Morsel, SimpleCookie, CookieError
from urllib.parse import unquote, quote
from enum import Enum
from typing import NamedTuple


mimetypes: Dict = ...
hop_by_hop_headers: Tuple = ...
http_methods: Tuple = ...


class UploadedFile:
    filename: str
    value: bytes
    def __init__(self, upfile) -> None: ...


class HttpStatus(Enum):
    class Status(NamedTuple):
        code: int
        reason: str

    @staticmethod
    def of(code: int) -> 'HttpStatus': ...

    OK: Status = ...
    Created: Status = ...
    Accepted: Status = ...
    NoContent: Status = ...
    MovedPermanently: Status = ...
    Found: Status = ...
    SeeOther: Status = ...
    NotModified: Status = ...
    TemporaryRedirect: Status = ...
    BadRequest: Status = ...
    Unauthorized: Status = ...
    Forbidden: Status = ...
    NotFound: Status = ...
    MethodNotAllowed: Status = ...
    NotAcceptable: Status = ...
    Conflict: Status = ...
    Gone: Status = ...
    PreconditionFailed: Status = ...
    UnsupportedMediaType: Status = ...
    UnprocessableEntity: Status = ...
    UnavailableForLegalReasons: Status = ...
    InternalServerError: Status = ...


class Cookie:
    name: str
    value: str
    expires: Optional[int]
    path: str
    domain: Optional[str]
    secure: bool
    httponly: bool

    def __init__(self, name:str, value:str, expires:int=None, path:str='/',
                 domain:str=None, secure:bool=False, httponly:bool=False) -> None: ...
    def dumps(self) -> str: ...


def parse_cookie(http_cookie)->Dict[str, str]: ...


# lessweb framework exceptions
class NeedParamError(Exception):
    def __init__(self, query: str, doc: str) -> None: ...
    def __repr__(self): ...
    def __str__(self): ...


class BadParamError(Exception):
    def __init__(self, query: str, error: str) -> None: ...
    def __repr__(self): ...
    def __str__(self): ...


class NotFoundError(Exception):
    def __init__(self, methods=None) -> None: ...
    def __repr__(self): ...
    def __str__(self): ...


def header_name_of_wsgi_key(wsgi_key) -> str: ...
def wsgi_key_of_header_name(header_name) -> str: ...



from setuptools import setup
import os


name = 'lessweb-stubs'


def find_stub_files():
    result = []
    for root, dirs, files in os.walk(name):
        for file in files:
            if file.endswith('.pyi'):
                if os.path.sep in root:
                    sub_root = root.split(os.path.sep, 1)[-1]
                    file = os.path.join(sub_root, file)
                result.append(file)
    return result


setup(
    name = name,
    version='0.1',
    description='lessweb stubs「嘞是web」',
    long_description='\nREADME: https://github.com/qorzj/lessweb\n\n'
                     'Cookbook: http://lessweb.org',
    url='https://github.com/qorzj/lessweb',
    author='qorzj',
    author_email='inull@qq.com',
    license='MIT License',
    classifiers=[],
    install_requires=[
       'mypy>=0.660',
       'typing-extensions>=3.6.5'
    ],
    keywords='lessweb web web.py stubs',
    py_modules=[],
    packages = ['lessweb-stubs'],
    package_data={'lessweb-stubs': find_stub_files()},  # cp -r lessweb-stubs ~/dev/venv37/lib/python3.7/site-packages/
)


from typing import *


class AnySub:
    def __init__(self, core: Type) -> None: ...


def generic_origin(t): ...
def generic_args(t): ...
def optional_core(t): ...
def collection_core(t: Type): ...
def issubtyping(cls: Type, parent: Type) -> bool: ...



from typing import Type, TypeVar, Any, Tuple, List
from abc import abstractmethod, ABCMeta


T = TypeVar('T')


class Bridge(metaclass=ABCMeta):
    bridges: List[Type['Bridge']]
    dist: Type
    @abstractmethod
    def __init__(self, source) -> None: ...
    @abstractmethod
    def to(self)->Any: ...
    def init_for_cast(self, bridges, dist=None) -> None: ...
    @classmethod
    def inspect(cls)->Tuple[Type, Type]: ...
    def cast(self, object: Any, source_type: Type, dist: Type[T]) -> T: ...


def assert_valid_bridge(bridge) -> None: ...
